GameplaySettings{
		usepuzzlegrid = false,
		--puzzlerows = 7,
		--puzzlecols = 3,
        greypercent=0.2,
        colorcount=3,
        usetraffic = true,
        automatic_traffic_collisions = false, -- the game shouldn't check for block collisions since we'll be doing that ourselves in this script
        jumpmode="none",

		--track generation settings
		gravity=-.45, -- even without jumping the gravity setting is (a little bit) relevant. It's used in generating the track to sculpt it steep enough to allow jumps
        playerminspeed = 0.3,--so the player is always moving somewhat
        playermaxspeed = 2.8,--2.5
        minimumbestjumptime = 2.5,--massage the track until a jump of at least this duration is possible
        uphilltiltscaler = 1,--set to 1 for normal track. higher for steeper
        downhilltiltscaler = 1,--set to 1 for normal track. higher for steeper
        uphilltiltsmoother = 0.03,
        downhilltiltsmoother = 0.06,
        useadvancedsteepalgorithm = true,--set false for a less extreme track
        alldownhill = false,
		--end track generation settings
}

SetSkinProperties{
	lanedividers={-2.5,2.5},
	shoulderlines={-7.5,7.5},
	trackwidth = 8,
	prefersteep = false
}

function deepcopy(orig) -- a pretty standard lua function for making a copy of a table
    local orig_type = type(orig)
    local copy
    if orig_type == 'table' then
        copy = {}
        for orig_key, orig_value in next, orig, nil do
            copy[deepcopy(orig_key)] = deepcopy(orig_value)
        end
        setmetatable(copy, deepcopy(getmetatable(orig)))
    else -- number, string, boolean, etc
        copy = orig
    end
    return copy
end

player={
	score=0,
	prevInput={},
	iPrevRing=0,
	hasFinishedScoringPrevRing=false,
	uniqueName = "Player",
	num=1,
	prevFirstBlockCollisionTested = 1,
	chainCount = 0,
	maxChainCount = 0,
	pos = {0,0,0},
	fuel = 250,
	posCosmetic = {0,0,1.5},
	controller = "mouse",
	points = 0 -- used for accumulating points this player earns at each match collection. temp var
}

function CompareJumpTimes(a,b) --used to sort the track nodes by jump duration
	return a.jumpairtime > b.jumpairtime
end

powernodes = powernodes or {}

track = track or {}
powernodes = powernodes or {}
function OnTrackCreated(theTrack)--track is created before the traffic
	track = theTrack --store a global copy of the track to maybe use later
	local songMinutes = track[#track].seconds / 60

	for i=1,#track do
		track[i].jumpedOver = false -- if this node was jumped over by a higher proiority jump
		track[i].origIndex = i
	end

	--find the best jumps path in this song
	local strack = deepcopy(track)
	table.sort(strack, CompareJumpTimes)

	for i=1,#strack do
		if strack[i].jumpairtime >= 3 then --only consider jumps of at least this amount of air time
			if not track[strack[i].origIndex].jumpedOver then
				local flightPathClear = true
				local jumpEndSeconds = strack[i].seconds + strack[i].jumpairtime + 10
				for j=strack[i].origIndex, #track do --make sure a higher priority jump doesn't happen while this one would be airborne
					if track[j].seconds <= jumpEndSeconds then
						if track[j].jumpedOver then
							flightPathClear = false
						end
					else
						break
					end
				end
				if flightPathClear then
					if #powernodes < songMinutes then -- allow about one power node per minute of music
						powernodes[#powernodes+1] = strack[i].origIndex
						jumpEndSeconds = strack[i].seconds + strack[i].jumpairtime + 10
						for j=strack[i].origIndex, #track do
							if track[j].seconds <= jumpEndSeconds then
								track[j].jumpedOver = true --mark this node as jumped over (a better jump took priority) so it is not marked as a powernode
							else
								break
							end
						end
					end
				end
			end
		end
	end
end

lanespace = 3.0
half_lanespace = 1.5
ylanespace = 2.0
half_ylanespace = 1.0
lane_margin = 0.25

blocks = blocks or {}
blockNodes = blockNodes or {}
blockOffsets = blockOffsets or {}
blockColors = blockColors or {}

function OnTrafficCreated(theTraffic)
	half_lanespace = lanespace / 2

	traffic = theTraffic

	local minimapMarkers = {}
		for j=1,#powernodes do --insert powernodes into the traffic
		local prev = 2
		for i=prev, #traffic do
			if traffic[i].impactnode >= powernodes[j] then
				if traffic[i].impactnode == powernodes[j] then
					traffic[i].powerupname = "powerpellet"
					traffic[i].type = 101 -- replace the block already at this node with a power pellet. 101 as a type doesn't mean anything to the game, but the script uses it
				else
					table.insert(traffic, i, {powerupname="powerpellet", type=101, impactnode=powernodes[j], chainstart=powernodes[j], chainend=powernodes[j], lane=0, strafe=0, strength=10})
				end
				prev = i
				table.insert(minimapMarkers, {tracknode=powernodes[j], startheight=0, endheight=fif(j==1, 15, 11), color=fif(j==1, {233,233,233}, nil) })
				break
			end
		end
	end
	AddMinimapMarkers(minimapMarkers)
	local counter = 0
	local i = 1
    while i <= #traffic do
		if counter ==0 then -- we remove half the traffic, because it would be too hard otherwise
		if traffic[i].type <3 then
			table.remove(traffic, i)
		end
			counter = 1
		else
			counter = 0
			local lane = math.random(-2,2)
			traffic[i].lane = lane
			local ylane = traffic[i].type
			if traffic[i].type ==101 then
			ylane=0
			elseif traffic[i].type >3 then
			ylane = math.random(0,2)
			end
			local strafe = lane*lanespace
			local ystrafe = ylane*ylanespace
			local offset = {strafe,0.5+ystrafe,0}
			traffic[i].v3offset = {strafe,0.5+ystrafe,0}
			traffic[i].chainstart = traffic[i].impactnode -- These two lines remove the line leading up to a block, they look ugly if the block is raised.
			traffic[i].chainend = traffic[i].impactnode
			local block = {}
			block.lane = lane
			block.ylane = ylane
			block.hidden = false
			block.tested = false
			block.type = traffic[i].type
			blocks[#blocks+1]=block
			blockNodes[#blockNodes+1] = traffic[i].impactnode
			blockOffsets[#blockOffsets+1] = offset
			blockColors[#blockColors+1] = track[traffic[i].impactnode].color
			i = i + 1
		end
    end

    return traffic -- when you return a traffic table from this function the game will read and apply any changes you made
end


function OnSkinLoaded()-- called after OnTrafficCreated. The skin script has loaded content.
	CreateClone{name=player.uniqueName, prefabName="Vehicle", attachToTrackWithNodeOffset=-1, transform={pos=player.pos}}
	HideBuiltinPlayerObjects()
	SetGlobalScore{score=0,showdelta=false}
	SetCamera{ -- calling this function (even just once) overrides the camera settings from the skin script
		nearcam={
			pos={0,3.0,-3.50475},
			rot={0,0,0},
			strafiness = 0.9
		},
		farcam={
			pos={0,3.0,-5},
			rot={0,0,0},
			strafiness = 0.9
		}
	}
end

iCurrentRing = 0 --Update function keeps this current
blocksToHide = {}
score = 0
stealthy = true
stealth_text = "(S)"
function Collide(ystrafe)
	local strafe = player.pos[1]
	local playerLane = 0;

	local absStrafe = math.abs(strafe)

		for i=2,0,-1 do
			if absStrafe > ((lanespace * i) + half_lanespace) then
				playerLane = i+1
				break
			end
		end

	if strafe < 0 then
		playerLane = -playerLane
	end
	local in_ylane = {}
	in_ylane[0] = (ystrafe<=half_ylanespace+lane_margin)
	in_ylane[1] = (math.abs(ystrafe-half_ylanespace) <= ylanespace + lane_margin)
	in_ylane[2] = (ystrafe>=ylanespace + half_ylanespace - lane_margin)

	local maxRing = iCurrentRing + 2
	local foundFirst = false
	for i=player.prevFirstBlockCollisionTested,#blockNodes do
		if not blocks[i].tested then
			if blockNodes[i] <= maxRing then
				if not foundFirst then
					player.prevFirstBlockCollisionTested = i
					foundFirst = true
				end

				if blockNodes[i] <= math.floor(iCurrentRing) then
					if not blocks[i].hidden then
						if (blocks[i].lane == playerLane and in_ylane[blocks[i].ylane]) then
							blocksToHide[#blocksToHide+1] = i
							blocks[i].hidden = true
							if blocks[i].type == 101 then
								score = score + 5000
								player.fuel = player.fuel + 250
								player.chainCount = player.chainCount + 24
								PlaySound{name="levelup"}
							elseif blocks[i].type == 5 then
									if stealthy then
										stealthy = false
										stealth_text = ""
										PlaySound{name="landcrash"} -- play crash sound when they lose stealth
									end 
							player.chainCount = -1
							score = score - 1000
							elseif blocks[i].ylane == 1 then
								score = score + 250*(1+0.01*player.chainCount)
							else
								score = score + 100*(1+0.01*player.chainCount)
							end
							PlaySound{name="hit"}
							player.chainCount = player.chainCount + 1
							if player.chainCount > player.maxChainCount then
								player.maxChainCount = player.chainCount
							end
							SetGlobalScore{score=score,showdelta=true}
							--SetScoreboardNote{text=stealth_text.."Chain Bonus: "..tostring(player.chainCount).."%", color={195,195,195}, secondsvisible=0}
							SendCommand{command="HoverUp"} -- causes the hovering vehicle to bounce up a bit
						else -- we didn't hit the block.
							if blocks[i].type ~= 5 then
							player.chainCount = 0
							--SetScoreboardNote{text=stealth_text.."Chain Bonus: "..tostring(player.chainCount).."%", color={195,195,195}, secondsvisible=0}
							end
						end
					end
					blocks[i].tested = true
				end
			else
				break --stop the loop once we get to a block way past the player
			end
			--end
		end
	end
	
end

function InsertLoopyLoop(track, apexNode, circumference)
    local halfSize = math.floor(circumference / 2)

    if (apexNode < halfSize) or ((apexNode + halfSize) > #track) then
    	return track
    end

    local startRing = math.max(1,apexNode - halfSize)
    local endRing = math.min(#track, apexNode + halfSize)
    --local endRingOriginalPosition = rings[endRing].pos;
    local span = endRing - startRing
    local startTilt = track[startRing].tilt
    local endOriginalTilt = track[endRing].tilt
    local tiltDeltaOverEntireLoop = -360 + (endOriginalTilt - startTilt)
    local pan = track[startRing].pan

    local panRate = 30 / circumference

    for i = startRing+1, endRing do
        track[i].tilt = startTilt + tiltDeltaOverEntireLoop * ((i - startRing) / span)
        pan = pan + panRate -- pan just a little while looping to make sure it doesn't run into itself
        track[i].pan = pan
        --local pan = track[i].pan
    end

    local panRejoinSpan = 175
    local panRejoinNode = endRing + panRejoinSpan
    local straightToFinish = false
    if panRejoinNode > #track then
    	straightToFinish = true
    else
    	local panPerNode = (track[panRejoinNode].pan - pan) / panRejoinSpan
    	for i=endRing+1, panRejoinNode do
    		pan = pan + panPerNode
    		track[i].pan = pan
    	end
    end

    local tiltDeltaCascade = track[endRing].tilt - endOriginalTilt;
    for i = endRing + 1, #track do
        track[i].tilt = track[i].tilt + tiltDeltaCascade
        if straightToFinish then
        	track[i].pan = pan
        end
    end

    return track
end

function OnRequestTrackReshaping(theTrack) -- put a loop at each powerpellet to make them easier to see coming
	track = theTrack
	for i=1,#powernodes do
			local size = 65 + 70 * math.max(1,(track[powernodes[i]].jumpairtime / 10))
			track = InsertLoopyLoop(track, math.max(1,powernodes[i]-size/3), size)
	end

	return track
end
function OnRequestLoadObjects()

	SetBlocks{
		powerups={
			powerpellet={mesh = "powerpellet.obj",
			shader = "RimLight",
			texture = "powerpellet.png"}
		}
	}	
end
mouseSpeed = .35
afterburn=0
flag=0
f=0
count=0
function Update(dt, tracklocation, playerstrafe, input) --called every frame
	if dt>0 then --don't move when the game is paused
	iCurrentRing = math.floor(tracklocation)
	local mouseInput = input["mouse"]
	local mouseHorizontal = mouseInput["x"]
	if count>0 then
	count=count-dt
	end
			player.pos[1] = math.max(-7, math.min(7, player.pos[1] + mouseHorizontal * mouseSpeed))
			if player.fuel>0 then
			if mouseInput["LMB"] then
			if (player.pos[2] ~= -1.4) and (player.pos[2] ~= 5) then
			player.fuel = player.fuel - .2*60*dt
			else afterburn=0
			end
			flag=1
			count=2
			elseif mouseInput["RMB"] then
			if (player.pos[2] ~= -1.4) and (player.pos[2] ~= 5) then
			player.fuel = player.fuel - .2*60*dt
			else afterburn=0
			end
			flag=-1
			count=0
			else flag=0
			if count<=0 then
			player.pos[2] = math.max(-1.4, math.min(5, player.pos[2] -dt/3))
			end
			end
			SetScoreboardNote{text=stealth_text.."Chain Bonus: "..tostring(player.chainCount).."%  Fuel: "..tostring(math.ceil(player.fuel)).." ", color={195,195,195}, secondsvisible=0}
			else player.fuel = 0 
			flag=0
			if count<=0 then
			player.pos[2] = math.max(-1.4, math.min(5, player.pos[2] -dt/3))
			end
			SetScoreboardNote{text=stealth_text.."Chain Bonus: "..tostring(player.chainCount).."%  Fuel: "..tostring(math.ceil(player.fuel)).." ", color={255,0,0}, secondsvisible=0}
			end
			
		if flag>0 then
		afterburn=math.min(1.25,afterburn+dt/(0.016/dt))
		elseif flag<0 then
		afterburn=math.max(-1.25,afterburn-dt/(0.016/dt))
		else
		if afterburn<0 then
		afterburn=math.min(0,afterburn+dt/(0.05/dt))
		elseif afterburn>0 then
		afterburn=math.max(0,afterburn-dt/(0.05/dt))
		end
		end
		player.pos[2] = math.max(-1.4, math.min(5, player.pos[2]+afterburn))
	player.posCosmetic[1] = player.pos[1]
	player.posCosmetic[2] = player.pos[2]
	SendCommand{command="SetTransform", name=player.uniqueName, param={pos=player.posCosmetic}}
	
	blocksToHide = {}
	local ystrafe = player.pos[2]
	--SetScoreboardNote{text="Position: "..tostring(player.posCosmetic[2]).." Mouse: "..tostring(mouseVertical), color={195,195,195}, secondsvisible=0.2}
	Collide(ystrafe)
	if #blocksToHide > 0 then
		HideTraffic(blocksToHide)
	end
	end
end

function OnRequestFinalScoring()
	chainBonus = player.chainCount*100
	local stealthBonus = 0
	if stealthy then 
		stealthBonus = math.floor(score * .3) 
	end
	return {
		rawscore = score,
		bonuses = {
			"Stealth Bonus:"..stealthBonus,
			"Maximum Chain Length: ".. tostring(player.maxChainCount),
			"End Chain Bonus: " .. tostring(player.chainCount) .. "X100"
		}
		,finalscore = score + chainBonus +stealthBonus
	}
end