--[[==================CREDITS======================]]--
--DeathByNukes Variours functions and models
--Sergalicious for the rails [skin named Viz]
--Roznick ring textures [skin named Geometricspace Revamped]
--SimonEvo for helping out with the skybox
--[[==================CREDITS======================]]--

--Feel free to use these options on your own skins too.
--NOTE these settings will be wiped everytime this skin gets updated if you are planning on making changes make sure to backup the changes.
options = {
cameramode = "static",			-- W.I.P "static" "zoomout" "dynamic" support for higher camera angles in the works.

colormode = "default",			-- "default" "invert" "classic"	different color sets

skyupdate = .005,				-- How fast the skybox moves

highspeed = true,				-- enables the layers 			

layers = 6						-- how many layers the rails have

}

--=========================Common.LuaInc=========================--
do
local pairs, ipairs, next, type = pairs, ipairs, next, type
local insert, concat = table.insert, table.concat

math.tau = math.pi * 2

function fif(test, if_true, if_false) if test then return if_true else return if_false end end

function sign(num)
	if num < 0 then return -1 end
	return 1
end

function createEvent()
	local listeners = {}
	return function(listener)
		insert(listeners, listener)
	end,
	function(...)
		for i=1,#listeners do listeners[i](...) end
	end
end


-- reverse sorts a number indexed table
function flipArray(t)
	local L = 1
	local R = #t
	while L < R do
		t[L], t[R] = t[R], t[L]
		L = L + 1
		R = R - 1
	end
	return t
end
-- shallow copies a number indexed table in reverse order
function copyFlipArray(t)
	local n = {}
	for i=#t,1,-1 do insert(n, t[i]) end
	return n
end

function concatTable(t, concat)
	for k,v in pairs(concat) do t[k] = v end
	return t
end

-- copy everything from the base table to the overrides table except for things that already exist there
-- works recursively for sub-tables
do
	local _Inherit
	function Inherit(base, overrides)
		for k,v in pairs(base) do
			over_v = overrides[k]
			if over_v == nil then
				overrides[k] = v
			elseif type(over_v) == "table" and type(v) == "table" then
				overrides[k] = _Inherit(v, over_v)
			end
		end
		return overrides
	end
	_Inherit = Inherit
end

function clones(value, count)
	local tbl = {}
	for i=1,count do tbl[i] = value end
	return unpack(tbl)
end
function clonesArray(value, count)
	local tbl = {}
	for i=1,count do tbl[i] = value end
	return tbl
end

white = {1, 1, 1}
black = {0, 0, 0}
function greyscale(amount, alpha) return {amount,amount,amount,alpha} end -- can omit alpha
function normalizeColor(c)
	if c[1] and c[2] and c[3] then return c end
	return {c.r, c.g, c.b, c.a}
end
function colorAlpha(c, a)
	c = {c[1] or c.r, c[2] or c.g, c[3] or c.b, a}
	if a ~= 0 and (c[1] > 1 or c[2] > 1 or c[3] > 1) then
		c[1], c[2], c[3] = c[1]/255, c[2]/255, c[3]/255
	end
	return c
end

-- for iterating over a table with the keys sorted alphabetically
-- http://www.lua.org/pil/19.3.html
do
	local sort = table.sort
	function pairsByKeys (t, f)
		local a = {}
		for n in pairs(t) do insert(a, n) end
		sort(a, f)
		local i = 0      -- iterator variable
		local iter = function ()   -- iterator function
			i = i + 1
			if a[i] == nil then return nil
			else return a[i], t[a[i]]
			end
		end
		return iter
	end
end

-- http://lua-users.org/wiki/TableUtils
do
	local string, tostring = string, tostring
	local _tableValToStr, _tableKeyToStr, _tableToString
	function tableValToStr ( v )
		if "string" == type( v ) then
			v = string.gsub( v, "\n", "\\n" )
			if string.match( string.gsub(v,"[^'\"]",""), '^"+$' ) then
				return "'" .. v .. "'"
			end
			return '"' .. string.gsub(v,'"', '\\"' ) .. '"'
		else
			return "table" == type( v ) and _tableToString( v ) or
				tostring( v )
		end
	end
	function tableKeyToStr ( k )
		if "string" == type( k ) and string.match( k, "^[_%a][_%a%d]*$" ) then
			return k
		else
			return "[" .. _tableValToStr( k ) .. "]"
		end
	end
	function tableToString( tbl )
		local result, done = {}, {}
		for k, v in ipairs( tbl ) do
			insert( result, _tableValToStr( v ) )
			done[ k ] = true
		end
		for k, v in pairs( tbl ) do
			if not done[ k ] then
				insert( result,
					_tableKeyToStr( k ) .. "=" .. _tableValToStr( v ) )
			end
		end
		return "{" .. concat( result, "," ) .. "}"
	end
	_tableValToStr, _tableKeyToStr, _tableToString = tableValToStr, tableKeyToStr, tableToString

	local pairsByKeys = pairsByKeys
	-- modified version of tableToString by DBN
	function tableToPrettyString( tbl )
		local result, done = {}, {}
		for k, v in ipairs( tbl ) do
			insert( result, "" )
			insert( result, _tableValToStr( v ) )
			done[ k ] = true
		end
		local indent = 0
		for k, v in pairsByKeys( tbl, function(a,b) return tostring(a) < tostring(b) end ) do
			if not done[ k ] then
				local key = _tableKeyToStr( k )
				if string.len(key) > indent then indent = string.len(key) end
				insert( result, key )
				insert( result, " = " .. _tableValToStr( v ) )
			end
		end
		--if indent ~= 0 then
			local last = #result - 1
			for i=1,last,2 do
				result[i] = result[i] .. string.rep(" ", indent - string.len(result[i]))
				if i ~= last then result[i+1] = result[i+1] .. ",\n\t" end
			end
		--end
		return "{\n\t" .. concat( result ) .. "\n}"
	end
end

do
	local setmetatable, getmetatable = setmetatable, getmetatable
	local _deepCopy
	function deepCopy(orig) -- a pretty standard lua function for making a copy of a table
		local orig_type = type(orig)
		local copy
		if orig_type == 'table' then
			copy = {}
			for orig_key, orig_value in next, orig, nil do
				copy[_deepCopy(orig_key)] = _deepCopy(orig_value)
			end
			setmetatable(copy, _deepCopy(getmetatable(orig)))
		else -- number, string, boolean, etc
			copy = orig
		end
		return copy
	end
	_deepCopy = deepCopy
end

function shallowCopy(orig)
	local copy = {}
	for orig_key, orig_value in next, orig, nil do
		copy[orig_key] = orig_value
	end
	return copy
end

function tableKeys(table)
	local keys = {}
	for key in pairs(table) do insert(keys, key) end
	return keys
end

-- AS2 API functions are supposed to handle lua objects with "using (obj) {}" blocks or at least call .Dispose() on them but it doesn't. (for skins, anyways. some mod functions are getting patched)
-- we can use these functions to try to stave off "out of memory" errors as long as possible
function destroy(tbl)
	for k in pairs(tbl) do tbl[k] = nil end
end
function destroyRecursive(tbl)
	for k, v in pairs(tbl) do
		if type(v) == "table" then destroyRecursive(v) end
		tbl[k] = nil
	end
end


function makeCylinderThruster(radius, points, inside_out)
	local min, max, step
	if inside_out then min, max, step = 0, points-1, 1
	else min, max, step = points-1, 0, -1 end
	
	local shape = {}
	local tau = math.tau
	for i=min,max,step do
		local angle = tau * (i/points)
		insert(shape, {radius * math.cos(angle), radius * math.sin(angle), 0})
	end
	return shape
end

end
--=========================Common.LuaInc=========================--
--=========================Random Functions=========================--
jumping = PlayerCanJump()
function fif(test, if_true, if_false)
  if test then return if_true else return if_false end
end

hifi = GetQualityLevel() > 2 -- GetQualityLevel returns 1,2,or 3
function ifhifi(if_true, if_false)
  if hifi then return if_true else return if_false end
end
quality = GetQualityLevel4()
function ByQuality4(low,med,high,ultra)
	if quality < 2 then return low
	elseif quality <3 then return med
	elseif quality <4 then return high
	else return ultra end
end
skinvars = GetSkinProperties()
trackWidth = skinvars["trackwidth"]
fullsteep = jumping or skinvars.prefersteep
extraWidth = trackWidth - 5

function fif(test, if_true, if_false) if test then return if_true else return if_false end end
quality = GetQualityLevel() -- GetQualityLevel returns 1,2,or 3
function ifquality(if_hi, if_mid, if_low)
	if quality >= 3 then 
		return if_hi
	elseif quality == 2 then
		return if_mid
	else
		return if_low
	end
end
white = {1, 1, 1}
black = {0, 0, 0}
function greyscale(amount, alpha) return {amount,amount,amount,alpha} end -- can omit alpha
track = GetTrack()
	do
		local A = track[1].pos
		local min={x=A.x, y=A.y, z=A.z}
		local max={x=A.x, y=A.y, z=A.z}
		for i=2,#track do
			local N = track[i].pos
			for key, value in pairs(max) do
				if N[key] > max[key] then max[key] = N[key] end
			end
			for key, value in pairs(max) do
				if N[key] < min[key] then min[key] = N[key] end
			end
		end
		track_bounds = {min=min, max=max}
	end

	track_bounds.center = {
		x=(track_bounds.min.x + track_bounds.max.x) / 2,
		y=(track_bounds.min.y + track_bounds.max.y) / 2,
		z=(track_bounds.min.z + track_bounds.max.z) / 2
	}
if options.cameramode == "static" then
    options.cameramode = {
        pos={0,5,-4},
        rot={28,0,0},
        strafefactor = 0.75,
        pos2={0,5,-4},
        rot2={28,0,0},
        strafefactorFar = 1,
        transitionspeed = 1,
        puzzleoffset=-0.85,
        puzzleoffset2=-1.75,
    }
elseif options.cameramode == "zoomout" then
    options.cameramode = {
        pos             = {0,14.375,-11.5},
        rot             = {28,0,0},
        strafefactorFar = .75,
        pos2            = {0,14.375,-11.5},
        rot2            = {28,0,0},
        strafefactorFar = .75,
        transitionspeed = 3,
        puzzleoffset    = -0.95,
        puzzleoffset2   = -1.5,
    }
elseif options.cameramode == "dynamic"  then
    options.cameramode = {
        pos = {0,2.85,-1.5},
        rot = {19,1,0},
        strafefactorFar = .9,
        pos2 = {0,14.375,-11.5},
        rot2 = {30,0,0},
        strafefactorFar = .9,
        transitionspeed = 1.5,
        puzzleoffset    = -0.65,
        puzzleoffset2   = -1.5,
    }
end    
--=========================Random Functions=========================--

--==================================================================--SCENE--=================================================================--
	SetScene{
	glowpasses = ifhifi(4,4),
	glowspread = ifhifi(0.3,0.3),
	radialblur_strength = ifhifi(0,0),
	watertype = 1,
	water = jumping, --only use the water cubes in wakeboard mode
	watertint = {r=255,g=255,b=255,a=255},
	watertexture = "WaterCubesBlue_BlackTop_WhiteLowerTier.png",
	hide_default_background_verticals = true,
	widewater = false, --sets water to 16.25 width
	towropes = jumping,--use the tow ropes if jumping
	airdebris_flashsizescaler = .2,
	airdebris_count = ifhifi(0,0),
	airdebris_density = ifhifi(1,1),
	airdebris_texture = "hit2.jpg",
	useblackgrid=false,
	use_intro_swoop_cam	= false,
	crease_strength=ifhifi(-47,0),
	closecam_near=.05,
	closecam_far=22 --bringing this in closer than the default (50) improves the ship's shadow quality
}

SetBlockFlashes{
	texture="hit2.jpg",
	sizescaler = 0,
	sizescaler_missed = 0
}

SetPuzzleGraphics{
	usesublayerclone = false,
	puzzlematchmaterial = {shader="Unlit/Transparent",texture="tileMatchingBars.png",aniso=9},
	puzzleflyupmaterial = {shader="VertexColorUnlitTintedAddFlyup",texture="tileMatchingBars3.png"},
	puzzlematerial = {shader="VertexColorUnlitTintedAlpha2",texture="tilesSquare.png",texturewrap="clamp",aniso=9, usemipmaps="false",shadercolors={_Color={255,255,255,255}}}
}
--==================================================================--SCENE--=================================================================--
--==================================================================--POST-EFFECTS--=================================================================--

AddPostEffect{
		depth= "foreground",
		builtin={
			name="AmplifyColor",
			lut = "LUTAC_cine1.png"
		}
	}
if options.airdebris then
	radialBlurEffect = BuildMaterial{ -- background radial blur (seen mostly on star air debris trails)
		shader="PostRadialBlur",
		shadersettings={_Amount=.015, _Center={.5,.5,0}}
	}

	AddPostEffect{
		depth="background",
		material = radialBlurEffect
	}
end	
	randompost = BuildMaterial{ 
		shader="Hidden/SSAO Pro V2",
		shadersettings={_Amount=.015, _Center={.5,.5,0}}
	}

	AddPostEffect{
		depth="background",
		material = randompost
	}
	anotherrandompost = BuildMaterial{ 
		shader="Hidden/GlowCompose",
		shadersettings={colour = 1}
	}

	AddPostEffect{
		depth="background",
		material = anotherrandompost
	}
--==================================================================--POST-EFFECTS--=================================================================--

--==================================================================--BLOCKS--=================================================================--

SetBlocks{
		maxvisiblecount = 500,
		colorblocks={
			mesh 	= "DBN_Assets/block.obj",
			shader 	= "Rim Light",
			shadercolors = {
					_AlphaGlowTint = greyscale(3, 3),
					_Color = greyscale(1/10),
					_RimColor = greyscale(4/4),
					_BorderAdd = greyscale(1),
					_BorderMult = greyscale(3),
				},
			textures = "DBN_assets/block.png",
			height 	= 0,
			float_on_water = false,
			scale 	= {1,0.75,1},
			reflect = false
		},
		greyblocks={
			mesh 	= 	"DBN_assets/block.obj",
			shader 	= 	"VertexColorUnlitTinted",
			texture = 	"DBN_assets/block.png"
		},
		chainspans={
			material = chainspanMaterial,
		},
		
powerups={
	powerpellet={
		mesh = "Powerpellet.obj",
		shader = "VertexColorUnlitTinted",
		shadersettings = fif(hifi, {_Brightness=2.0}, {_Brightness=3.5}),
		texture= "white.png",
		scale = {1.75,1.75,1.75},
		shadercolors= {_Color = {colorsource="highwayinverted", scaletype="intensity", minscaler=1, maxscaler=2}}
		}
	}	

}
--==================================================================--BLOCKS--=================================================================--

--=================================================================---PLAYER---=================================================================--

        local ship_material = BuildMaterial{
 
            renderqueue = 2000,
 
           shader = "MatCap/Vertex/PlainBrightGlow",
			shadersettings={
				_GlowScaler = 9,
				_Brightness = 2/3,
			},
			shadercolors={
				_Color =     { colorsource="highway", scaletype="intensity", minscaler=1, maxscaler=1 },
				_GlowColor = { colorsource="highway", scaletype="intensity", minscaler=1, maxscaler=1 },
			},
			textures={
				_MatCap="matcapchrome.jpg",
				_Glow="auvBGHD.png",
			},	
 
 		}
 
        local player = {
 
            cameramode = "third",
 
            camthird=options.cameramode,
           
 
 
         vehicle={
				min_hover_height= 0.11,
				max_hover_height = 1.4,
				use_water_rooster = false,
				smooth_tilting = false,
				smooth_tilting_speed = 10,
				smooth_tilting_max_offset = -20,
			shader = "MatCap/Vertex/PlainBrightGlow",
			shadersettings={
				_GlowScaler = 1,
				_Brightness = 1/3,
			},
			shadercolors={
				_Color =     { colorsource="highway", scaletype="intensity", minscaler=1, maxscaler=1 },
				_GlowColor = { colorsource="highway", scaletype="intensity", minscaler=1, maxscaler=1 },
			},
			textures={
				_MatCap="matcapchrome.jpg",
				_Glow="auvBGHD.png",
			},	
			mesh = BuildMesh{
					mesh="ninjamono.obj",
					barycentricTangents = true, --for use with wireframe shaders
					calculateNormals = false,
					submeshesWhenCombining = true
				},
				reflect = false,
				layer = 13,
			reflect = false,
			renderqueue = 2000,
			scale = {x=1,y=1,z=1},
			thrusters = {
					crossSectionShape={
						{-.35,-.35, 0},
						{-.5 ,0   , 0},
						{-.35, .35, 0},
						{0   , .5 , 0},
						{ .35, .35, 0},
						{ .5 ,0   , 0},
						{ .35,-.35, 0},
					},
					crossSectionShape = makeCylinderThruster(0.5, ifquality(10,7,7)),
                    perShapeNodeColorScalers = clonesArray(1,20), -- mandatory. can have more entries than the number of cross section points
                    shader="VertexColorUnlitTintedAdd",
                    renderqueue = 3998,
                    layer = 13,
                    shadercolors={
                        _Color = { colorsource="highway", scaletype="intensity", minscaler=1, maxscaler= 1, },
                    },
                    colorscaler = {
                        close = 1.0,
                        far = 0.5,
                    },
                    sizefalloff = 1,
                    minrenderedsize = 0,
                    extrusions = 25,
                    stretch = -0.1191,
                    updateseconds = 0.025,
				},
				childobjects={
				{
					mesh = "ninjamonoW.obj",
					shader="VertexColorUnlitTinted",
					shadercolors={
                        _Color = { colorsource="highway", scaletype="intensity", minscaler=1, maxscaler= 1, },
                    },
					texture="White.png"
				}
				}
			},
		}
		

		do
			
			local center, right, left =
				{0,.46,-1.28},
				{.175, 0.28*0.75, -2.04*0.75},
				{-.175, 0.28*0.75, -2.04*0.75}

			player.vehicle.thrusters.instances = {
				{ pos=center, scale={0.60*0.75, 0.60*0.75, 0.6*0.75} },
				{ pos=center, scale={0.45*0.75, 0.45*0.75, 0.8*0.75} },
				{ pos=center, scale={0.30*0.75, 0.30*0.75, 0.8*0.75} },
				{ pos=center, scale={0.10*0.75, 0.10*0.75, 0.8*0.75} },

				{ pos=right,  scale={0.3*0.75, 0.3*0.75, 0.66*0.75} },
				{ pos=right,  scale={0.1*0.75, 0.1*0.75, 0.66*0.75} },

				{ pos=left,   scale={0.3*0.75, 0.3*0.75, 0.66*0.75} },
				{ pos=left,   scale={0.1*0.75, 0.1*0.75, 0.66*0.75} }
			}
			
		end
		
		
		SetPlayer(player)
--=================================================================---PLAYER---=================================================================--
--=================================================================---COLORS---=================================================================--
if options.colormode == "default" then
		SetTrackColors{	
		{255, 160,   0}, -- orange (low intensity)
		{248,  38,   0}, -- red
		{174,   0, 255}, -- purple
		{  0, 127, 255}, -- blue
		{134, 201, 255},} -- bright blue (high intensity)
elseif options.colormode == "invert" then
		SetTrackColors{	
		{134, 201, 255}, -- bright blue 
		{  0, 127, 255}, -- blue
		{174,   0, 255}, -- purple
		{248,  38,   0}, -- red
		{255, 160,   0},} -- orange
elseif options.colormode == "classic" then
		SetTrackColors{
		{214, 0, 254},
    	{0, 176, 255},
    	{0, 184, 0},
    	{255, 255, 0},
		{255, 0, 0},}
else
options.colormode = {}
end				
--=================================================================---COLORS---=================================================================--
----=================================================---MISC---=================================================---
wakeStrafe = 7.5
SetWake{ --setup the spray coming from the two pulling "boats"
	height = 0
}
-- set up the look of the 2D rings the game places at intense sections of the track
SetRings{
	texture="portal_onblack.png",
	shader="VertexColorUnlitTintedAddSmooth",
	layer=13,
	
	size=22, -- Scale 0 because we want no regular rings
	percentringed=0.5,	
	
	-- Data for air rings
	airtexture="black.png",
	airshader="VertexColorUnlitTintedAlpha",
	useairrings = false,
	airsize=0 --don't want air rings
}

Nodes = {}
Offsets = {}
RotationSpeeds = {}
RotationSpeeds2 = {}
RotationSpeeds4 = {}
Nodes[#Nodes+1] = #track
Offsets[#Offsets+1] = {0,30,0}
RotationSpeeds[#RotationSpeeds+1] = {20,20,20}
RotationSpeeds2[#RotationSpeeds2+1] = {20,-10,20}
RotationSpeeds4[#RotationSpeeds4+1] = {-20,20,20}
CreateObject{
	name="EndCookie",
	visible=false,
	gameobject={
		transform={scale={scaletype="intensity",min={200,200,200},max={380,380,380}}},
		--layer=13,
		mesh="quad.obj",
		shader = "VertexColorUnlitTintedAddSmooth",
			texture="Diamond_Test.png",
			shadercolors={
			_Color={colorsource="highway", scaletype="intensity", minscaler=3, maxscaler=2},
		},
	}
}


CreateObject{
	name="EndCookie1",
	visible=false,
	gameobject={
		transform={scale={scaletype="intensity",min={200,200,200},max={380,380,380}}},
		mesh="quad1.obj",
		shader = "VertexColorUnlitTintedAddSmooth",
			texture="Diamond_Test.png",
			shadercolors={
			_Color={colorsource="highway", scaletype="intensity", minscaler=3, maxscaler=2},
		},
	}
}
CreateObject{
	name="EndCookie2",
	visible=false,
	gameobject={
		transform={scale={scaletype="intensity",min={200,200,200},max={380,380,380}}},
		--layer=13,
		mesh="quad2.obj",
		shader = "VertexColorUnlitTintedAddSmooth",
			texture="Diamond_Test.png",
			shadercolors={
			_Color={colorsource="highway", scaletype="intensity", minscaler=3, maxscaler=2},
		},
	}
}
CreateObject{
	name="EndCookie3",
	visible=false,
	gameobject={
		transform={scale={scaletype="intensity",min={200,200,200},max={380,380,380}}},
		--layer=13,
		mesh="quad3.obj",
		shader = "VertexColorUnlitTintedAddSmooth",
			texture="Diamond_Test.png",
			shadercolors={
			_Color={colorsource="highway", scaletype="intensity", minscaler=3, maxscaler=2},
		},
	}
}
CreateObject{
	name="EndCookie4",
	visible=false,
	gameobject={
		transform={scale={scaletype="intensity",min={200,200,200},max={380,380,380}}},
		mesh="quad4.obj",
		shader = "VertexColorUnlitTintedAddSmooth",
			texture="Diamond_Test.png",
			shadercolors={
			_Color={colorsource="highway", scaletype="intensity", minscaler=3, maxscaler=2},
		},
	}
}
CreateObject{
	name="EndCookie5",
	visible=false,
	gameobject={
		transform={scale={scaletype="intensity",min={200,200,200},max={380,380,380}}},
		mesh="quad5.obj",
		shader = "VertexColorUnlitTintedAddSmooth",
			texture="Diamond_Test.png",
			shadercolors={
			_Color={colorsource="highway", scaletype="intensity", minscaler=3, maxscaler=2},
		},
	}
}
BatchRenderEveryFrame{
	prefabName="EndCookie", --tell the game to render these prefabs in a batch (with Graphics.DrawMesh) every frame
	locations=Nodes,
	rotateWithTrack=false,
	maxDistanceShown=40000000,
	rotationspeeds = RotationSpeeds,
	testAndHideIfCollideWithTrack=false --if true, it checks each render location against a ray down the center of the track for collision. Any hits are not rendered
}
BatchRenderEveryFrame{
	prefabName="EndCookie1", --tell the game to render these prefabs in a batch (with Graphics.DrawMesh) every frame
	locations=Nodes,
	rotateWithTrack=false,
	maxDistanceShown=40000000,
	rotationspeeds = RotationSpeeds,
	testAndHideIfCollideWithTrack=false --if true, it checks each render location against a ray down the center of the track for collision. Any hits are not rendered
}
BatchRenderEveryFrame{
	prefabName="EndCookie2", --tell the game to render these prefabs in a batch (with Graphics.DrawMesh) every frame
	locations=Nodes,
	rotateWithTrack=false,
	maxDistanceShown=40000000,
	rotationspeeds = RotationSpeeds2,
	testAndHideIfCollideWithTrack=false --if true, it checks each render location against a ray down the center of the track for collision. Any hits are not rendered
}
BatchRenderEveryFrame{
	prefabName="EndCookie3", --tell the game to render these prefabs in a batch (with Graphics.DrawMesh) every frame
	locations=Nodes,
	rotateWithTrack=false,
	maxDistanceShown=40000000,
	rotationspeeds = RotationSpeeds2,
	testAndHideIfCollideWithTrack=false --if true, it checks each render location against a ray down the center of the track for collision. Any hits are not rendered
}
BatchRenderEveryFrame{
	prefabName="EndCookie4", --tell the game to render these prefabs in a batch (with Graphics.DrawMesh) every frame
	locations=Nodes,
	rotateWithTrack=false,
	maxDistanceShown=40000000,
	rotationspeeds = RotationSpeeds4,
	testAndHideIfCollideWithTrack=false --if true, it checks each render location against a ray down the center of the track for collision. Any hits are not rendered
}
BatchRenderEveryFrame{
	prefabName="EndCookie5", --tell the game to render these prefabs in a batch (with Graphics.DrawMesh) every frame
	locations=Nodes,
	rotateWithTrack=false,
	maxDistanceShown=40000000,
	rotationspeeds = RotationSpeeds4,
	testAndHideIfCollideWithTrack=false --if true, it checks each render location against a ray down the center of the track for collision. Any hits are not rendered
}
RotationSpeeds1 = {}
RotationSpeeds3 = {}
RotationSpeeds5 = {}

RotationSpeeds1[#RotationSpeeds1+1] = {-20,-20,-20}
RotationSpeeds3[#RotationSpeeds3+1] = {-20,10,-20}
RotationSpeeds5[#RotationSpeeds5+1] = {20,-20,-20}
CreateObject{
	name="1EndCookie",
	visible=false,
	gameobject={
		transform={scale={scaletype="intensity",min={380,380,380},max={200,200,200}}},
		mesh="quad.obj",
		shader = "VertexColorUnlitTintedAddSmooth",
		texture="cube_test.png",
			shadercolors={
			_Color={colorsource="highway", scaletype="intensity", minscaler=2, maxscaler=3},
		},
	}
}
CreateObject{
	name="1EndCookie1",
	visible=false,
	gameobject={
		transform={scale={scaletype="intensity",min={380,380,380},max={200,200,200}}},
		mesh="quad1.obj",
		shader = "VertexColorUnlitTintedAddSmooth",
		texture="cube_test.png",
			shadercolors={
			_Color={colorsource="highway", scaletype="intensity", minscaler=2, maxscaler=3},
		},
	}
}
CreateObject{
	name="1EndCookie2",
	visible=false,
	gameobject={
		transform={scale={scaletype="intensity",min={380,380,380},max={200,200,200}}},
		mesh="quad2.obj",
		shader = "VertexColorUnlitTintedAddSmooth",
			texture="cube_test.png",
			shadercolors={
			_Color={colorsource="highway", scaletype="intensity", minscaler=2, maxscaler=3},
		},
	}
}
CreateObject{
	name="1EndCookie3",
	visible=false,
	gameobject={
		transform={scale={scaletype="intensity",min={380,380,380},max={200,200,200}}},
		mesh="quad3.obj",
		shader = "VertexColorUnlitTintedAddSmooth",
			texture="cube_test.png",
			shadercolors={
			_Color={colorsource="highway", scaletype="intensity", minscaler=2, maxscaler=3},
		},
	}
}
CreateObject{
	name="1EndCookie4",
	visible=false,
	gameobject={
		transform={scale={scaletype="intensity",min={380,380,380},max={200,200,200}}},
		mesh="quad4.obj",
		shader = "VertexColorUnlitTintedAddSmooth",
			texture="cube_test.png",
			shadercolors={
			_Color={colorsource="highway", scaletype="intensity", minscaler=2, maxscaler=3},
		},
	}
}
CreateObject{
	name="1EndCookie5",
	visible=false,
	gameobject={
		transform={scale={scaletype="intensity",min={380,380,380},max={200,200,200}}},
		mesh="quad5.obj",
		shader = "VertexColorUnlitTintedAddSmooth",
			texture="cube_test.png",
			shadercolors={
			_Color={colorsource="highway", scaletype="intensity", minscaler=2, maxscaler=3},
		},
	}
}
BatchRenderEveryFrame{
	prefabName="1EndCookie", --tell the game to render these prefabs in a batch (with Graphics.DrawMesh) every frame
	locations=Nodes,
	rotateWithTrack=false,
	maxDistanceShown=40000000,
	rotationspeeds = RotationSpeeds1,
	testAndHideIfCollideWithTrack=false --if true, it checks each render location against a ray down the center of the track for collision. Any hits are not rendered
}
BatchRenderEveryFrame{
	prefabName="1EndCookie1", --tell the game to render these prefabs in a batch (with Graphics.DrawMesh) every frame
	locations=Nodes,
	rotateWithTrack=false,
	maxDistanceShown=40000000,
	rotationspeeds = RotationSpeeds1,
	testAndHideIfCollideWithTrack=false --if true, it checks each render location against a ray down the center of the track for collision. Any hits are not rendered
}
BatchRenderEveryFrame{
	prefabName="1EndCookie2", --tell the game to render these prefabs in a batch (with Graphics.DrawMesh) every frame
	locations=Nodes,
	rotateWithTrack=false,
	maxDistanceShown=40000000,
	rotationspeeds = RotationSpeeds3,
	testAndHideIfCollideWithTrack=false --if true, it checks each render location against a ray down the center of the track for collision. Any hits are not rendered
}
BatchRenderEveryFrame{
	prefabName="1EndCookie3", --tell the game to render these prefabs in a batch (with Graphics.DrawMesh) every frame
	locations=Nodes,
	rotateWithTrack=false,
	maxDistanceShown=40000000,
	rotationspeeds = RotationSpeeds3,
	testAndHideIfCollideWithTrack=false --if true, it checks each render location against a ray down the center of the track for collision. Any hits are not rendered
}
BatchRenderEveryFrame{
	prefabName="1EndCookie4", --tell the game to render these prefabs in a batch (with Graphics.DrawMesh) every frame
	locations=Nodes,
	rotateWithTrack=false,
	maxDistanceShown=40000000,
	rotationspeeds = RotationSpeeds5,
	testAndHideIfCollideWithTrack=false --if true, it checks each render location against a ray down the center of the track for collision. Any hits are not rendered
}
BatchRenderEveryFrame{
	prefabName="1EndCookie5", --tell the game to render these prefabs in a batch (with Graphics.DrawMesh) every frame
	locations=Nodes,
	rotateWithTrack=false,
	maxDistanceShown=40000000,
	rotationspeeds = RotationSpeeds5,
	testAndHideIfCollideWithTrack=false --if true, it checks each render location against a ray down the center of the track for collision. Any hits are not rendered
}

function Update(dt, trackLocation, playerStrafe, playerJumpHeight, intensity)
	local fft = GetSpectrum(); -- GetLogSpectrum();
	local morphweights = {
						3.5 * (fft[1]+fft[2]+fft[3]),
						1.5 * (fft[4]+fft[5]+fft[6]),
						1.5 * (fft[7]+fft[8]+fft[9]),
						1.5 * (fft[10]+fft[11]+fft[12]),
						1.5 * (fft[13]+fft[14]+fft[15]+fft[16])
					}

	UpdateMeshMorphWeights{mesh=spikeMesh, weights=morphweights}
	UpdateMeshMorphWeights{mesh=ballMesh, weights=morphweights}
	local morphweightsreversed = {morphweights[5], morphweights[4], morphweights[3], morphweights[2], morphweights[1]}
	UpdateMeshMorphWeights{mesh=squareMesh, weights=morphweightsreversed}
end
----=================================================---MISC---=================================================---
----=================================================---SKYBOX---=================================================---
CreateObject{
name="lvf",
railoffset=0,
gameobject={
pos={x=0,y=0,z=0},
mesh="222.obj",
shader="VertexColorUnlitTintedMultiplyNoDepthWrite",
shadercolors={
_Color = {colorsource="highway", scaletype="intensity", minscaler=1, maxscaler=1},
},
texture="White.png",
Color={colorsource="highway", scaletype="intensity", minscaler=1, maxscaler=1},
scale = {x=1000,y=1000,z=1000}
}
}

randomSky = math.random(0,28)
SetSkybox{
    skysphere = "s" .. randomSky ..".png"
    }
   
skyUpdateCounter = 0
function Update(dt)
    skyUpdateCounter = skyUpdateCounter + dt
    if(skyUpdateCounter > options.skyupdate) then--don't update sky colors every frame
    randomSky=randomSky+1
    skyUpdateCounter = 0
        SetSkybox{
    skysphere = "s" ..randomSky..".png"
    }
   
if(randomSky == 29) then
        randomSky = -1
        end
    end
end
----=================================================---SKYBOX---=================================================---
----=================================================---RAILS---=================================================---
shaderset = {_Color = {colorsource="aurora", scaletype="intensity", minscaler=0.13, maxscaler=2.5}}
for i=1,fif(options.highspeed,options.layers,1) do
auroraExtent = ifhifi(0.06, 0.03)
auroraHeight = -1.9 * (auroraExtent / 1.85)

CreateRail{--left aurora right
	positionOffset={
	x=1.62,
	y=0.5},
	crossSectionShape={
	{x=-1.0,y=-.0},
	{x=-auroraExtent,y=-.255}},
	perShapeNodeColorScalers={
	0,
	.2,
	.2,
	0},
	colorMode="aurora",
	color = {r=255,g=255,b=255},
	flatten=false,
    layer=11,
	texture="White.png",
	shader="VertexColorUnlitTintedAddSmooth",
	shadercolors=shaderset
	}

CreateRail{--left aurora left
	positionOffset={
	x=-1.45,
	y=0.5},
	crossSectionShape={
	{x=-1.0,y=-.0},
	{x=-auroraExtent,y=-.255}},
	perShapeNodeColorScalers={
	0,
	.2,
	.2,
	0},
	colorMode="aurora",
	color = {r=255,g=255,b=255},
	flatten=false,
    layer=11,
	texture="White.png",
	shader="VertexColorUnlitTintedAddSmooth",
	shadercolors=shaderset
	}


CreateRail{--right aurora right
	positionOffset={
		x=1.40,
		y=0.5},
	crossSectionShape={
	{x=auroraExtent,y=-.255},
	{x=1.0,y=-.0}},
	perShapeNodeColorScalers={
	.2,
	0,
	0,
	.2},
	colorMode="aurora",
	color = {r=255,g=255,b=255},
	flatten=false,
    layer=11,
	texture="White.png",
	shader="VertexColorUnlitTintedAddSmooth",
	shadercolors=shaderset
	}
	
CreateRail{--right aurora left
	positionOffset={
	x=-1.65,
	y=0.5},
	crossSectionShape={
	{x=auroraExtent,y=-.255},
	{x=1.0,y=-.0}},
	perShapeNodeColorScalers={
	.2,
	0,
	0,
	.2},
	colorMode="aurora",
	color = {r=255,g=255,b=255},
	flatten=false,
    layer=11,
	texture="White.png",
	shader="VertexColorUnlitTintedAddSmooth",
	shadercolors=shaderset
	}
	
CreateRail{--rollercoaster overhead rail
	positionOffset={
	x=0,
	y=18},
	crossSectionShape={
	{x=-1.0,y=1.0},
	{x=1.0,y=1.0},
	{x=1.0,y=-1.0},
	{x=-1.0,y=-1.0}},
	perShapeNodeColorScalers={
	1,
	1,
	1,
	1},
	colorMode="aurora",
	color = {r=255,g=255,b=255},
	flatten=false,
	fullfuture = true,
	texture="White.png",
	shader="VertexColorUnlitTintedAddSmooth"
	}	

		---aurora versions


CreateRail{-- lower left aurora  right
	positionOffset={
	x=17.62-extraWidth,
	y=-15.0},
	crossSectionShape={
	{x=-1.0,y=-.0},
	{x=-auroraExtent,y=-.255}},
	perShapeNodeColorScalers={
	0,
	.2,
	.2,
	0},
	colorMode="aurora",
	color = {r=255,g=255,b=255},
	flatten=false,
    layer=11,
	texture="White.png",
	shader="VertexColorUnlitTintedAddSmooth",
	shadercolors=shaderset
	}

CreateRail{-- lower left aurora left
	positionOffset={
	x=-17.45-extraWidth,
	y=-15.0},
	crossSectionShape={
	{x=-1.0,y=-.0},
	{x=-auroraExtent,y=-.255}},
	perShapeNodeColorScalers={
	0,
	.2,
	.2,
	0},
	colorMode="aurora",
	color = {r=255,g=255,b=255},
	flatten=false,
    layer=11,
	texture="White.png",
	shader="VertexColorUnlitTintedAddSmooth",
	shadercolors=shaderset
	}


CreateRail{--lower right aurora right
	positionOffset={
	x=17.40+extraWidth,
	y=-15.0},
	crossSectionShape={
	{x=auroraExtent,y=-.255},
	{x=1.0,y=-.0}},
	perShapeNodeColorScalers={
	.2,
	0,
	0,
	.2},
	colorMode="aurora",
	color = {r=255,g=255,b=255},
	flatten=false,
    layer=11,
	texture="White.png",
	shader="VertexColorUnlitTintedAddSmooth",
	shadercolors=shaderset
	}
	
CreateRail{--  lower right aurora left
	positionOffset={
	x=-17.65+extraWidth,
	y=-15.0},
	crossSectionShape={
	{x=auroraExtent,y=-.255},
	{x=1.0,y=-.0}},
	perShapeNodeColorScalers={
	.2,
	0,
		0,
		.2},
	colorMode="aurora",
	color = {r=255,g=255,b=255},
	flatten=false,
    layer=11,
	texture="White.png",
	shader="VertexColorUnlitTintedAddSmooth",
	shadercolors=shaderset
	}

end
----=================================================---RAILS---=================================================---